@@ -1,3 +1,4 @@
+from discord.channel import TextChannel
 from .cogs import BaseCallable, CogCommand, CogMessageCommand, CogSubCommandGroup, ListeningComponent
 from .components import Component, ComponentType
 from .slash.errors import NoAsyncCallback
@@ -5,9 +6,9 @@ from .errors import MissingListenedComponentParameters, WrongType
 from .slash.tools import ParseMethod, cache_data, format_name, handle_options, handle_thing
 from .slash.http import create_global_command, create_guild_command, delete_global_command, delete_guild_command, delete_guild_commands, edit_global_command, edit_guild_command, get_command, get_command_permissions, get_global_commands, get_guild_commands, delete_global_commands, get_id, update_command_permissions
 from .slash.types import AdditionalType, CommandType, ContextCommand, MessageCommand, OptionType, SlashCommand, SlashOption, SlashSubcommand, UserCommand
-from .tools import MISSING, _or, get_index, setup_logger
+from .tools import MISSING, _none, _or, get_index, setup_logger, get
 from .http import jsonifyMessage, BetterRoute, send_files
-from .receive import Interaction, Message, PressedButton, SelectedMenu, SlashedContext, WebhookMessage, SlashedCommand, SlashedSubCommand, getMessage
+from .receive import ComponentContext, Interaction, Message, PressedButton, SelectedMenu, SlashedContext, WebhookMessage, SlashedCommand, SlashedSubCommand, getMessage
 from .override import override_dpy as override_it
 
 import discord
@@ -172,6 +173,7 @@ class Slash():
             await interaction.defer(self.auto_defer[1])
         self._discord.dispatch("interaction_received", interaction)
 
+
         #region basic commands
         if data["data"]["type"] == CommandType.SLASH and not (data["data"].get("options") and data["data"]["options"][0]["type"] in [OptionType.SUB_COMMAND, OptionType.SUB_COMMAND_GROUP]):
             x = self.commands.get(data["data"]["name"])
@@ -299,50 +301,7 @@ class Slash():
         }
         own_guild_ids = [x.id for x in self._discord.guilds]
         
-        #region gather commands
-        commands = self.commands.copy()
-        for _base in self.subcommands:
-            # get first base
-            for _sub in self.subcommands[_base]:
-                # get second base/command
-                sub = self.subcommands[_base][_sub]
-                # when command has subcommand groups
-                if isinstance(sub, dict):
-                    for _group in self.subcommands[_base][_sub]:
-                        # the subcommand group
-                        group = self.subcommands[_base][_sub][_group]
-                        # if there's already a base command
-                        if commands.get(_base) is not None:
-                            # Check if base already has an option with the subs name
-                            index = get_index(commands[_base].options, _sub, lambda x: getattr(x, "name"))
-                            # if first base_name already exists
-                            if index > -1:
-                                # add to sub options
-                                base_ops = commands[_base].options
-                                base_ops[index].options += [group.to_option()]
-                                commands[_base].options = base_ops
-                            # if not exists
-                            else:
-                                # create sub option + group option
-                                commands[_base].options += [SlashOption(OptionType.SUB_COMMAND_GROUP, _sub, options=[group.to_option()])]
-                        # if no base command
-                        else:
-                            # create base0 command together with base1 option and groupcommand option
-                            commands[_base] = SlashCommand(None, _base, MISSING, [
-                                    SlashOption(OptionType.SUB_COMMAND_GROUP, _sub, options=[group.to_option()])
-                                ],
-                                guild_ids=group.guild_ids, default_permission=group.default_permission, guild_permissions=group.guild_permissions)
-                # if is basic subcommand
-                else:
-                    # If base exists
-                    if commands.get(_base) is not None:
-                        commands[_base].options += [sub.to_option()]
-                    else:
-                        # create base0 command with name option
-                        commands[_base] = SlashCommand(None, _base, options=[sub.to_dict()], guild_ids=sub.guild_ids, default_permission=sub.default_permission, guild_permissions=sub.guild_permissions)
-        #endregion
-
-
+        commands = self.gather_commands()
         async def guild_stuff(command, guild_ids):
             """Adds the command to the guilds"""
             for x in guild_ids:
@@ -438,6 +397,48 @@ class Slash():
         logging.debug("getting guild commands in " + str(guild_id))
         return await get_guild_commands(self._discord, guild_id)
     
+    def gather_commands(self) -> Dict[str, SlashCommand]:
+        commands = self.commands.copy()
+        for _base in self.subcommands:
+            # get first base
+            for _sub in self.subcommands[_base]:
+                # get second base/command
+                sub = self.subcommands[_base][_sub]
+                # when command has subcommand groups
+                if isinstance(sub, dict):
+                    for _group in self.subcommands[_base][_sub]:
+                        # the subcommand group
+                        group = self.subcommands[_base][_sub][_group]
+                        # if there's already a base command
+                        if commands.get(_base) is not None:
+                            # Check if base already has an option with the subs name
+                            index = get_index(commands[_base].options, _sub, lambda x: getattr(x, "name"))
+                            # if first base_name already exists
+                            if index > -1:
+                                # add to sub options
+                                base_ops = commands[_base].options
+                                base_ops[index].options += [group.to_option()]
+                                commands[_base].options = base_ops
+                            # if not exists
+                            else:
+                                # create sub option + group option
+                                commands[_base].options += [SlashOption(OptionType.SUB_COMMAND_GROUP, _sub, options=[group.to_option()])]
+                        # if no base command
+                        else:
+                            # create base0 command together with base1 option and groupcommand option
+                            commands[_base] = SlashCommand(None, _base, MISSING, [
+                                    SlashOption(OptionType.SUB_COMMAND_GROUP, _sub, options=[group.to_option()])
+                                ],
+                                guild_ids=group.guild_ids, default_permission=group.default_permission, guild_permissions=group.guild_permissions)
+                # if is basic subcommand
+                else:
+                    # If base exists
+                    if commands.get(_base) is not None:
+                        commands[_base].options += [sub.to_option()]
+                    else:
+                        # create base0 command with name option
+                        commands[_base] = SlashCommand(None, _base, options=[sub.to_dict()], guild_ids=sub.guild_ids, default_permission=sub.default_permission, guild_permissions=sub.guild_permissions)
+        return commands
 
     async def create_command(self, command):
         """
@@ -643,7 +644,7 @@ class Slash():
         else:
             await self.create_command(command)
         self._set_command(old_name, command)
-            
+   
     def _get_command(self, name, typ: Literal["slash", 1, "user", 2, "message", 3]) -> SlashCommand:
         typ = CommandType.from_string(typ)
         if typ == CommandType.SLASH:
@@ -740,12 +741,12 @@ class Slash():
         logging.info("nuked all commands")
 
 
-    def add_command(self, name, callback=None, description=MISSING, options=MISSING, guild_ids=MISSING, default_permission=True, guild_permissions=MISSING, api=False) -> Union[None, Coroutine]:
+    def add_command(self, name=MISSING, callback=None, description=MISSING, options=MISSING, guild_ids=MISSING, default_permission=True, guild_permissions=MISSING, api=False) -> Union[None, Coroutine]:
         """
         Adds a new slashcommand
 
         name: :class:`str`
-            1-32 characters long name
+            1-32 characters long name; default MISSING
 
             .. note::
 
@@ -856,6 +857,9 @@ class Slash():
             """
             self.add_command(name, callback, description, options, guild_ids, default_permission, guild_permissions)
         return wrapper
+    def add_subcommand(self, base_names, name=MISSING, callback=None, description=MISSING, options=MISSING, guild_ids=MISSING, default_permission=True, guild_permissions=MISSING):
+        command = SlashSubcommand(callback, base_names, name, description, options, guild_ids=guild_ids, default_permission=default_permission, guild_permissions=guild_permissions)
+        self._add_to_cache(command)
     def subcommand(self, base_names, name=MISSING, description=MISSING, options=[], guild_ids=MISSING, default_permission=True, guild_permissions=MISSING):
         """
         A decorator for a subcommand group
@@ -1190,6 +1194,8 @@ class Components():
             await interaction.defer(self.auto_defer[1])
         self._discord.dispatch("interaction_received", interaction)
 
+        self._discord.dispatch("component", ComponentContext(self._discord._connection, data, user, msg))
+
         # Handle auto_defer
         if int(data["data"]["component_type"]) == 2:
             for x in msg.buttons:
@@ -1254,10 +1260,20 @@ class Components():
         :type: :class:`~Message`
         """
 
-        if not isinstance(channel, (discord.TextChannel, int, str)):
-            raise WrongType("channel", channel, "discord.TextChannel")
+        if not isinstance(channel, (discord.TextChannel, int, str, discord.User)):
+            raise WrongType("channel", channel, ["discord.TextChannel", "discord.User"])
 
-        channel_id = channel.id if isinstance(channel, discord.TextChannel) else channel
+        channel_id = None
+        if isinstance(channel, discord.User):
+            if channel.dm_channel is None:
+                channel = await channel.create_dm()
+                channel_id = channel.id
+            else:
+                channel_id = channel.dm_channel
+        elif isinstance(channel, discord.TextChannel):
+            channel_id = channel.id
+        else: 
+            channel_id = channel
         payload = jsonifyMessage(content=content, tts=tts, embed=embed, embeds=embeds, nonce=nonce, allowed_mentions=allowed_mentions, reference=reference, mention_author=mention_author, components=components)
 
         route = BetterRoute("POST", f"/channels/{channel_id}/messages")
@@ -1268,9 +1284,9 @@ class Components():
         else:
             r = await send_files(route, files=_or(files, [file]), payload=payload, http=self._discord.http)
 
-        msg = Message(state=self._discord._get_state(), channel=channel, data=r)
-            
-        if delete_after is not None:
+        msg = Message(state=self._discord._connection, channel=channel, data=r)
+        
+        if not _none(delete_after):
             await msg.delete(delay=delete_after)
         
         return msg
